// GridInterface.hpp
//
//     Created on: 29. 03. 2018
//         Author: Jan Uhlik
//    Modified by:
//
// Copyright (c) 2017 Czech Technical University in Prague | Faculty of Information Technology. All rights reserved.
// Git repository: https://gitlab.fit.cvut.cz/algorithms-library-toolkit/automata-library

#ifndef ALIB2_GRIDINTERFACE_HPP
#define ALIB2_GRIDINTERFACE_HPP

#include <alib/set>
#include <alib/pair>
#include <alib/vector>
#include <functional>

#include "GridBase.hpp"

namespace grid {

template<typename TCoordinate, typename TEdge>
class GridInterface : public GridBase {
// ---------------------------------------------------------------------------------------------------------------------
 public:
  using coordinate_type = TCoordinate;
  using edge_type = TEdge;
  using node_type = ext::pair<TCoordinate, TCoordinate>;

// ---------------------------------------------------------------------------------------------------------------------

  virtual size_t nodeCount() const = 0;
  virtual size_t edgeCount() const = 0;

// ---------------------------------------------------------------------------------------------------------------------

  virtual ext::set<node_type> getNodes() const = 0;
  virtual ext::vector<TEdge> getEdges() const = 0;

// ---------------------------------------------------------------------------------------------------------------------

  virtual ext::set<node_type> successors(const node_type &n) const = 0;
  virtual ext::vector<TEdge> successorEdges(const node_type &n) const = 0;
  virtual ext::set<node_type> predecessors(const node_type &n) const = 0;
  virtual ext::vector<TEdge> predecessorEdges(const node_type &n) const = 0;

// ---------------------------------------------------------------------------------------------------------------------

  virtual std::string name() const = 0;

// ---------------------------------------------------------------------------------------------------------------------
};

// =====================================================================================================================

}

#endif //ALIB2_GRIDINTERFACE_HPP
